@@ -35,6 +35,7 @@
 #define PROPLUSX   0x4552
 #define VTX        0x4557
 #define I750TC     0x455A
+#define I770R      0x4651
 
 #define MAXRETRIES 2
 #define MAXDELAY   16
@@ -56,6 +57,7 @@
 typedef struct oceanic_atom2_device_t {
 	oceanic_common_device_t base;
 	dc_iostream_t *iostream;
+	unsigned int sequence;
 	unsigned int delay;
 	unsigned int bigpage;
 	unsigned char cache[256];
@@ -198,6 +200,19 @@ static const oceanic_common_version_t aeris_a300cs_version[] = {
 	{"AQUAI750 \0\0 2048"},
 };
 
+// Not 100% sure what the pattern is.
+// I've seen:
+//
+//   "AQUA770R 1A 0001"
+//   "AQUA770R 1A 0090"
+//
+// from the same dive computer. On other ones, it's
+// apparently the two middle digits that change, on
+// the i770R it might be all of them.
+static const oceanic_common_version_t aqualung_i770r_version[] = {
+	{"AQUA770R \0\0 \0\0\0\0"},
+};
+
 static const oceanic_common_version_t aqualung_i450t_version[] = {
 	{"AQUAI450 \0\0 2048"},
 };
@@ -487,6 +502,21 @@ static const oceanic_common_layout_t aeris_a300cs_layout = {
 	0, /* pt_mode_serial */
 };
 
+static const oceanic_common_layout_t aqualung_i770r_layout = {
+	0x440000, /* memsize */
+	0x40000, /* highmem */
+	0x0000, /* cf_devinfo */
+	0x0040, /* cf_pointers */
+	0x2000, /* rb_logbook_begin */
+	0x10000, /* rb_logbook_end */
+	16, /* rb_logbook_entry_size */
+	0x40000, /* rb_profile_begin */
+	0x440000, /* rb_profile_end */
+	0, /* pt_mode_global */
+	1, /* pt_mode_logbook */
+	0, /* pt_mode_serial */
+};
+
 static const oceanic_common_layout_t aqualung_i450t_layout = {
 	0x40000, /* memsize */
 	0, /* highmem */
@@ -570,7 +600,7 @@ oceanic_atom2_packet (oceanic_atom2_device_t *device, const unsigned char comman
 
 
 static dc_status_t
-oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int crc_size)
+oceanic_atom2_serial_transfer (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int crc_size)
 {
 	// Send the command to the device. If the device responds with an
 	// ACK byte, the command was received successfully and the answer
@@ -600,6 +630,200 @@ oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char comm
 	return DC_STATUS_SUCCESS;
 }
 
+/*
+ * The BLE GATT packet size is up to 20 bytes and the format is:
+ *
+ * byte 0: <0xCD>
+ *         Seems to always have this value. Don't ask what it means
+ * byte 1: <d 1 c s s s s s>
+ *          d=0 means "command", d=1 means "reply from dive computer"
+ *          1 is always set, afaik
+ *          c=0 means "last packet" in sequence, c=1 means "more packets coming"
+ *          sssss is a 5-bit sequence number for packets
+ * byte 2: <cmd seq>
+ *          starts at 0 for the connection, incremented for each command
+ * byte 3: <length of data>
+ *          1-16 bytes of data per packet.
+ * byte 4..n: <data>
+ */
+static dc_status_t
+oceanic_atom2_ble_write(oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize)
+{
+	unsigned char buf[20];
+	unsigned char cmd_seq = device->sequence;
+	unsigned char pkt_seq;
+
+	pkt_seq = 0;
+	while (csize) {
+		dc_status_t ret;
+		unsigned char status = 0x40;
+		unsigned int cpartial = csize;
+		if (cpartial > 16) {
+			cpartial = 16;
+			status |= 0x20;
+		}
+		buf[0] = 0xcd;
+		buf[1] = status | (pkt_seq & 31);
+		buf[2] = cmd_seq;
+		buf[3] = cpartial;
+		memcpy(buf+4, command, cpartial);
+		command += cpartial;
+		csize -= cpartial;
+		ret = dc_iostream_write(device->iostream, buf, 4+cpartial, NULL);
+		if (ret != DC_STATUS_SUCCESS)
+			return ret;
+		pkt_seq++;
+	}
+	return DC_STATUS_SUCCESS;
+}
+
+static dc_status_t
+oceanic_atom2_ble_read(oceanic_atom2_device_t *device, unsigned char **result_p, unsigned int *size_p)
+{
+	unsigned char *result = NULL;
+	unsigned int size = 0, allocated = 0;
+	unsigned char buf[20];
+	unsigned char cmd_seq = device->sequence;
+	unsigned char pkt_seq;
+	dc_status_t ret = DC_STATUS_SUCCESS;
+
+	pkt_seq = 0;
+	for (;;) {
+		unsigned char status, expect;
+		size_t transferred = 0;
+		ret = dc_iostream_read(device->iostream, buf, sizeof(buf), &transferred);
+		if (ret != DC_STATUS_SUCCESS)
+			break;
+
+		ret = DC_STATUS_IO;
+		if (transferred < 5 || transferred > 20) {
+			ERROR(device->base.base.context, "Odd BLE packet size %zd", transferred);
+			break;
+		}
+		if (buf[0] != 0xcd)
+			ERROR(device->base.base.context, "Odd first byte (got '%02x', expected 'cd'", buf[0]);
+
+		// Verify status byte
+		expect = 0xc0;
+		expect |= (pkt_seq & 31);
+		status = buf[1];
+		if ((status & ~0x20) != expect)
+			ERROR(device->base.base.context, "Odd status byte (got '%02x', expected '%02x'", buf[1], expect);
+
+		// Verify command sequence byte
+		expect = cmd_seq;
+		if (buf[2] != expect)
+			ERROR(device->base.base.context, "Odd cmd sequence byte (got '%02x', expected '%02x'", buf[2], expect);
+
+		// Verify length byte
+		expect = buf[3];
+		if (expect < 1 || expect > 16) {
+			ERROR(device->base.base.context, "Odd reply size byte (got %d, expected 1..16", buf[3]);
+			break;
+		}
+
+		if (transferred < 4+expect) {
+			ERROR(device->base.base.context, "Packet too small (got %zd bytes, expected at least %d bytes)", transferred, 4+expect);
+			break;
+		}
+
+		if (size + expect > allocated) {
+			unsigned int newsize = size + expect + 100;
+			unsigned char *newalloc = realloc(result, newsize);
+			if (!newalloc) {
+				ret = DC_STATUS_NOMEMORY;
+				break;
+			}
+			result = newalloc;
+			allocated = newsize;
+		}
+
+		memcpy(result + size, buf+4, expect);
+		size += expect;
+		pkt_seq++;
+
+		/* More packets? */
+		if (status & 0x20)
+			continue;
+
+		ret = DC_STATUS_SUCCESS;
+		break;
+	}
+
+	if (ret != DC_STATUS_SUCCESS) {
+		free(result);
+		size = 0;
+		result = NULL;
+	}
+	*result_p = result;
+	*size_p = size;
+	return ret;
+}
+
+/*
+ * Transfer a command and optionally read return data.
+ *
+ * NOTE! The NUL byte at the end of a command is a serial transfer thing,
+ * and we remove it. The correct thing to do would be to add it on the
+ * serial transfer side instead (or perhaps not send it at all, Jef says
+ * it may be historical), but right now I've tried to minimize the changes
+ * that the BLE transfer code made to the code, so instead this tries to
+ * just skip the extraneous byte.
+ */
+static dc_status_t
+oceanic_atom2_ble_transfer (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int crc_size)
+{
+	unsigned char buf[20];
+	unsigned char cmd_seq = device->sequence;
+	unsigned char pkt_seq;
+	dc_status_t ret = DC_STATUS_SUCCESS;
+	int retry = 3;
+
+	/*
+	 * The serial commands have a NUL byte at the end. It's bogus.
+	 * It should be added on the serial transfer side, not removed
+	 * here.
+	 */
+	if (csize > 1 && csize < 8 && !command[csize-1])
+		csize--;
+
+retry:
+	if (--retry < 0)
+		return ret;
+
+	ret = oceanic_atom2_ble_write(device, command, csize);
+	if (ret != DC_STATUS_SUCCESS)
+		return ret;
+
+	pkt_seq = 0;
+	if (answer) {
+		unsigned char *buf;
+		unsigned int size;
+		ret = oceanic_atom2_ble_read(device, &buf, &size);
+		if (ret != DC_STATUS_SUCCESS)
+			goto retry;
+		if (size > asize && buf[0] == ACK) {
+			memcpy(answer, buf+1, asize);
+			device->sequence++;
+		} else {
+			ERROR(device->base.base.context, "Result too small: got %d bytes, expected at least %d bytes", size, asize+1);
+			ret = DC_STATUS_IO;
+			goto retry;
+		}
+		free(buf);
+	}
+
+	return ret;
+}
+
+static dc_status_t
+oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int crc_size)
+{
+	if (dc_iostream_get_transport(device->iostream) == DC_TRANSPORT_BLE)
+		return oceanic_atom2_ble_transfer(device, command, csize, answer, asize, crc_size);
+
+	return oceanic_atom2_serial_transfer(device, command, csize, answer, asize, crc_size);
+}
 
 static dc_status_t
 oceanic_atom2_quit (oceanic_atom2_device_t *device)
@@ -636,15 +860,24 @@ oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, dc_iostream
 	// Set the default values.
 	device->iostream = iostream;
 	device->delay = 0;
+	device->sequence = 0;
 	device->bigpage = 1; // no big pages
 	device->cached_page = INVALID;
 	device->cached_highmem = INVALID;
 	memset(device->cache, 0, sizeof(device->cache));
 
 	// Get the correct baudrate.
-	unsigned int baudrate = 38400;
-	if (model == VTX || model == I750TC || model == PROPLUSX) {
+	unsigned int baudrate;
+	switch (model) {
+	case VTX:
+	case I750TC:
+	case PROPLUSX:
+	case I770R:
 		baudrate = 115200;
+		break;
+	default:
+		baudrate = 38400;
+		break;
 	}
 
 	// Set the serial communication protocol (38400 8N1).
@@ -661,21 +894,37 @@ oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, dc_iostream
 		goto error_free;
 	}
 
-	// Give the interface 100 ms to settle and draw power up.
+	// Set the DTR line.
+	status = dc_iostream_set_dtr (device->iostream, 1);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to set the DTR line.");
+		goto error_free;
+	}
+
+	// Clear the RTS line to reset the PIC inside the data cable as it
+	// may not have have been previously cleared. This ensures that the
+	// PIC will always start in a known state once RTS is set. Starting
+	// in a known default state is very important as the PIC won't
+	// respond to init commands unless it is in a default state.
+	status = dc_iostream_set_rts (device->iostream, 0);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to clear the RTS line.");
+		goto error_free;
+	}
+
+	// Hold RTS clear for a bit to allow PIC to reset.
 	dc_iostream_sleep (device->iostream, 100);
 
-	// Set the DTR/RTS lines.
-	status = dc_iostream_set_dtr(device->iostream, 1);
+	// Set the RTS line.
+	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the DTR line.");
-		return status;
-	}
-	status = dc_iostream_set_rts(device->iostream, 1);
-	if (status != DC_STATUS_SUCCESS) {
-		ERROR (context, "Failed to set the DTR line.");
-		return status;
+		ERROR (context, "Failed to set the RTS line.");
+		goto error_free;
 	}
 
+	// Give the interface 100 ms to settle and draw power up.
+	dc_iostream_sleep (device->iostream, 100);
+
 	// Make sure everything is in a sane state.
 	dc_iostream_purge (device->iostream, DC_DIRECTION_ALL);
 
@@ -733,6 +982,9 @@ oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, dc_iostream
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, aeris_a300cs_version)) {
 		device->base.layout = &aeris_a300cs_layout;
 		device->bigpage = 16;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, aqualung_i770r_version)) {
+		device->base.layout = &aqualung_i770r_layout;
+		device->bigpage = 16;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, aqualung_i450t_version)) {
 		device->base.layout = &aqualung_i450t_layout;
 	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_default_version)) {
@@ -794,9 +1046,55 @@ oceanic_atom2_device_keepalive (dc_device_t *abstract)
 	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
+	/* No answer: increment sequence number manually */
+	device->sequence++;
 	return DC_STATUS_SUCCESS;
 }
 
+/*
+ * The BLE communication sends a handshake packet that seems
+ * to be a passphrase based on the BLE name of the device
+ * (more specifically the serial number encoded in the name).
+ *
+ * The packet format is:
+ *    0xe5
+ *    < 8 bytes of passphrase >
+ *    one-byte checksum of the passphrase.
+ */
+static dc_status_t
+oceanic_atom2_send_ble_handshake(oceanic_atom2_device_t *device)
+{
+	unsigned char handshake[10] = { 0xe5, }, ack[1];
+	const char *bt_name = dc_iostream_get_name(device->iostream);
+
+	/*
+	 * Allow skipping the handshake if no name. But the download will
+	 * likely fail.
+	 *
+	 * The format of the name is something like 'FQ001124', where the
+	 * two first letters indicate the kind of device it is, and the
+	 * six digits are the serial number.
+	 *
+	 * Jef theorizes that 'FQ' in hexadecimal is 0x4651, which is
+	 * the model number of the i770R.
+	 */
+	if (!bt_name || strlen(bt_name) < 8)
+		return DC_STATUS_SUCCESS;
+
+	/* Turn ASCII numbers into just raw byte values */
+	for (int i = 0; i < 6; i++)
+		handshake[i+1] = bt_name[i+2] - '0';
+
+	/* Add simple checksum */
+	handshake[9] = checksum_add_uint8(handshake+1, 8, 0x00);
+
+	/*
+	 * .. and send it off.
+	 *
+	 * NOTE! We don't expect any data back, but we do want the ACK.
+	 */
+	return oceanic_atom2_ble_transfer(device, handshake, sizeof(handshake), ack, 0, 0);
+}
 
 dc_status_t
 oceanic_atom2_device_version (dc_device_t *abstract, unsigned char data[], unsigned int size)
@@ -817,7 +1115,10 @@ oceanic_atom2_device_version (dc_device_t *abstract, unsigned char data[], unsig
 
 	memcpy (data, answer, PAGESIZE);
 
-	return DC_STATUS_SUCCESS;
+	if (dc_iostream_get_transport(device->iostream) == DC_TRANSPORT_BLE)
+		rc = oceanic_atom2_send_ble_handshake(device);
+
+	return rc;
 }
 
 
@@ -938,6 +1239,9 @@ oceanic_atom2_device_write (dc_device_t *abstract, unsigned int address, const u
 		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
+		/* No answer, increment sequence number manually */
+		device->sequence++;
+
 		nbytes += PAGESIZE;
 		address += PAGESIZE;
 		data += PAGESIZE;
