@@ -89,6 +89,7 @@
 #define I550        0x4642
 #define I200        0x4646
 #define I100        0x464E
+#define I770R       0x4651
 
 #define NORMAL   0
 #define GAUGE    1
@@ -177,7 +178,8 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		parser->headersize = 5 * PAGESIZE;
 		parser->footersize = 0;
 	} else if (model == A300CS || model == VTX ||
-		model == I450T || model == I750TC) {
+		model == I450T || model == I750TC ||
+		model == I770R) {
 		parser->headersize = 5 * PAGESIZE;
 	} else if (model == PROPLUSX) {
 		parser->headersize = 3 * PAGESIZE;
@@ -319,6 +321,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case I450T:
 		case I750TC:
 		case PROPLUSX:
+		case I770R:
 			datetime->year   = (p[10]) + 2000;
 			datetime->month  = (p[8]);
 			datetime->day    = (p[9]);
@@ -421,6 +424,7 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	unsigned int ngasmixes = 0;
 	unsigned int o2_offset = 0;
 	unsigned int he_offset = 0;
+	unsigned int pO2_offset = 1;
 	if (mode == FREEDIVE) {
 		ngasmixes = 0;
 	} else if (parser->model == DATAMASK || parser->model == COMPUMASK) {
@@ -456,6 +460,13 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	} else if (parser->model == ZEN) {
 		o2_offset = header + 4;
 		ngasmixes = 2;
+	} else if (parser->model == PROPLUSX) {
+		o2_offset = 0x24;
+		ngasmixes = 4;
+	} else if (parser->model == I770R) {
+		o2_offset = 0x30;
+		ngasmixes = 4;
+		pO2_offset = 2;
 	} else {
 		o2_offset = header + 4;
 		ngasmixes = 3;
@@ -467,8 +478,8 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 	parser->mode = mode;
 	parser->ngasmixes = ngasmixes;
 	for (unsigned int i = 0; i < ngasmixes; ++i) {
-		if (data[o2_offset + i]) {
-			parser->oxygen[i] = data[o2_offset + i];
+		if (data[o2_offset + i*pO2_offset]) {
+			parser->oxygen[i] = data[o2_offset + i*pO2_offset];
 		} else {
 			parser->oxygen[i] = 21;
 		}
@@ -545,7 +556,7 @@ oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 			break;
 		case DC_FIELD_SALINITY:
 			if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I750TC) {
+				parser->model == I750TC || parser->model == I770R) {
 				if (data[0x18] & 0x80) {
 					water->type = DC_WATER_FRESH;
 				} else {
@@ -648,7 +659,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		unsigned int idx = 0x17;
 		if (parser->model == A300CS || parser->model == VTX ||
 			parser->model == I450T || parser->model == I750TC ||
-			parser->model == PROPLUSX)
+			parser->model == PROPLUSX || parser->model == I770R)
 			idx = 0x1f;
 		switch (data[idx] & 0x03) {
 		case 0:
@@ -703,7 +714,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == OC1C || parser->model == OCI ||
 		parser->model == TX1 || parser->model == A300CS ||
 		parser->model == VTX || parser->model == I450T ||
-		parser->model == I750TC || parser->model == PROPLUSX) {
+		parser->model == I750TC || parser->model == PROPLUSX ||
+		parser->model == I770R) {
 		samplesize = PAGESIZE;
 	}
 
@@ -732,7 +744,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 	if (have_pressure) {
 		unsigned int idx = 2;
 		if (parser->model == A300CS || parser->model == VTX ||
-			parser->model == I750TC)
+			parser->model == I750TC || parser->model == I770R)
 			idx = 16;
 		pressure = array_uint16_le(data + parser->header + idx);
 		if (pressure == 10000)
@@ -785,7 +797,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				tank = 0;
 				pressure = (((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF);
 			} else if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I750TC) {
+				parser->model == I750TC || parser->model == I770R) {
 				// Tank pressure (1 psi) and number (one based index)
 				tank = (data[offset + 1] & 0x03) - 1;
 				pressure = ((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF;
@@ -881,7 +893,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					parser->model == XPAIR) {
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else if (parser->model == A300CS || parser->model == VTX ||
-					parser->model == I750TC || parser->model == PROPLUSX) {
+					parser->model == I750TC || parser->model == PROPLUSX ||
+					parser->model == I770R) {
 					temperature = data[offset + 11];
 				} else {
 					unsigned int sign;
@@ -922,7 +935,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else if (parser->model == TX1 || parser->model == A300CS ||
 					parser->model == VTX || parser->model == I750TC ||
-					parser->model == PROPLUSX)
+					parser->model == PROPLUSX || parser->model == I770R)
 					pressure = array_uint16_le (data + offset + 4);
 				else
 					pressure -= data[offset + 1];
@@ -971,7 +984,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			unsigned int decostop = 0, decotime = 0;
 			if (parser->model == A300CS || parser->model == VTX ||
 				parser->model == I450T || parser->model == I750TC ||
-				parser->model == PROPLUSX) {
+				parser->model == PROPLUSX || parser->model == I770R) {
 				decostop = (data[offset + 15] & 0x70) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x03FF;
 				have_deco = 1;
